package com.company.bpmdemo.core;

import com.haulmont.bpm.entity.ProcActor;
import com.haulmont.bpm.entity.ProcInstance;
import com.haulmont.bpm.entity.ProcRole;
import com.haulmont.cuba.core.global.CommitContext;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.Metadata;
import com.haulmont.cuba.security.entity.User;
import org.slf4j.Logger;
import org.springframework.stereotype.Component;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

@Component(ProcRolesHelper.NAME)
public class ProcRolesHelper {
    public static final String NAME = "bpmdemo_ProcRolesHelper";


    @Inject
    private Logger log;

    @Inject
    private DataManager dataManager;

    @Inject
    private Metadata metadata;

    public void fillProcRoles(String procRoleCode, String secRoleName, UUID bpmProcInstanceId) {
        ProcInstance procInstance = dataManager.load(ProcInstance.class)
                .id(bpmProcInstanceId)
                .view("procInstance-listener-view")
                .one();
        Optional<ProcRole> procRoleOpt = procInstance.getProcDefinition().getProcRoles().stream()
                .filter(procRole -> procRole.getCode().equals(procRoleCode))
                .findAny();
        if (!procRoleOpt.isPresent()) {
            log.error("ProcRole {} not found", procRoleCode);
            return;
        }
        List<User> usersWithSecRole = findUsersBySecRole(secRoleName);
        List<ProcActor> procActorsToCreate = new ArrayList<>();
        for (User user : usersWithSecRole) {
            ProcActor procActor = metadata.create(ProcActor.class);
            procActor.setUser(user);
            procActor.setProcInstance(procInstance);
            procActor.setProcRole(procRoleOpt.get());
            procActorsToCreate.add(procActor);
        }
        dataManager.commit(new CommitContext(procActorsToCreate));
    }

    private List<User> findUsersBySecRole(String secRoleName) {
        return dataManager.load(User.class)
                .query("select u from sec$User u join u.userRoles ur join ur.role r where r.name = :secRoleName")
                .parameter("secRoleName", secRoleName)
                .list();
    }
}